# ===============================================================================
# Copyright 2017 GustavoJunior
# ===============================================================================

import os
import re
import sys
import math
import time
import urllib
import urllib2
import tarfile
from datetime import datetime, timedelta

import web_tools


class StationNotFoundError(Exception):
    pass


class InvalidSatelliteError(Exception):
    pass


def connect_earth_explorer(usgs):
    cookies = urllib2.HTTPCookieProcessor()
    opener = urllib2.build_opener(cookies)
    urllib2.install_opener(opener)

    data = urllib2.urlopen("https://ers.cr.usgs.gov").read()
    m = re.search(r'<input .*?name="csrf_token".*?value="(.*?)"', data)
    if m:
        token = m.group(1)
    else:
        print "Error : CSRF_Token not found"
        # sys.exit(-3)

    params = urllib.urlencode(dict(username=usgs['account'], password=usgs['passwd'], csrf_token=token))
    request = urllib2.Request("https://ers.cr.usgs.gov/login", params, headers={})
    f = urllib2.urlopen(request)

    data = f.read()
    f.close()
    if data.find('You must sign in as a registered user to download data or place orders for USGS EROS products') > 0:
        print "Authentification failed"
        # sys.exit(-1)
    return


def download_chunks(url, output_dir, image):
    """ Downloads large files in pieces
  """
    try:
        req = urllib2.urlopen(url)
        # if downloaded file is html
        if req.info().gettype() == 'text/html':
            print "error : file is in html and not an expected binary file"
            lines = req.read()
            if lines.find('Download Not Found') > 0:
                raise TypeError
            else:
                with open("error_output.html", "w") as f:
                    f.write(lines)
                    print "result saved in ./error_output.html"
                    # sys.exit(-1)
        # if file too small
        total_size = int(req.info().getheader('Content-Length').strip())

        if total_size < 50000:
            print "Error: The file is too small to be a Landsat Image"
            print url
            # sys.exit(-1)
        print image, total_size
        total_size_fmt = sizeof_fmt(total_size)

        # download
        downloaded = 0
        CHUNK = 1024 * 1024 * 8
        with open(output_dir + '/' + image, 'wb') as fp:
            start = time.clock()
            print('Downloading {0} ({1}):'.format(image, total_size_fmt))
            while True:
                chunk = req.read(CHUNK)
                downloaded += len(chunk)
                done = int(50 * downloaded / total_size)
                sys.stdout.write('\r[{1}{2}]{0:3.0f}% {3}ps'
                                 .format(math.floor((float(downloaded)
                                                     / total_size) * 100),
                                         '=' * done,
                                         ' ' * (50 - done),
                                         sizeof_fmt((downloaded // (time.clock() - start)) / 8)))
                sys.stdout.flush()
                if not chunk: break
                fp.write(chunk)
    except urllib2.HTTPError, e:
        if e.code == 500:
            print 'error code 500: file doesnt exist'
            pass
        else:
            print "HTTP Error:", e.code, url
        return False
    except urllib2.URLError, e:
        print "URL Error:", e.reason, url
        return False

    return output_dir, image


def sizeof_fmt(num):
    for x in ['bytes', 'KB', 'MB', 'GB', 'TB']:
        if num < 1024.0:
            return "%3.1f %s" % (num, x)
            # num /= 1024.0


def unzip_image(tgzfile, outputdir):
    target_tgz = os.path.join(outputdir, tgzfile)
    if os.path.exists(target_tgz):
        print 'found tgs: {} \nunzipping...'.format(target_tgz)
        tfile = tarfile.open(target_tgz, 'r:gz')
        tfile.extractall(outputdir)
        print 'unzipped\ndeleting tgz: {}'.format(target_tgz)
        os.remove(target_tgz)
    else:
        raise NotImplementedError('Did not find download output directory to unzip...')
    return None


def get_credentials(usgs_path):
    with file(usgs_path) as f:
        (account, passwd) = f.readline().split(' ')
        if passwd.endswith('\n'):
            passwd = passwd[:-1]
        usgs = {'account': account, 'passwd': passwd}
        return usgs


def get_station_list_identifier(product):
    if product.startswith('LC8'):
        identifier = '12864'
        stations = ['LGN']
    elif product.startswith('LE7'):
        identifier = '3373'
        stations = ['EDC', 'SGS', 'AGS', 'ASN', 'SG1', 'CUB', 'COA']
    elif product.startswith('LT5'):
        identifier = '3119'
        stations = ['GLC', 'ASA', 'KIR', 'MOR', 'KHC', 'PAC',
                    'KIS', 'CHM', 'LGS', 'MGR', 'COA', 'MPS', 'CUB']
    else:
        raise NotImplementedError('Must provide valid product string...')

    return identifier, stations



def down_usgs_by_id(scene_id, output_dir, usgs_creds_txt):

    usgs_creds = get_credentials(usgs_creds_txt)
    connect_earth_explorer(usgs_creds)

    identifier, stations = get_station_list_identifier(scene_id)
    base_url = 'https://earthexplorer.usgs.gov/download/'
    tail_string = '{}/{}/STANDARD/EE'.format(identifier,scene_id)
    url = '{}{}'.format(base_url,tail_string)
    tgz_file = '{}.tgz'.format(scene_id)
    scene_dir = os.path.join(output_dir, scene_id)
    if not os.path.isdir(scene_dir):
	os.mkdir(scene_dir)
	download_chunks(url,scene_dir,tgz_file)
	print 'image {}'.format(os.path.join(scene_dir,tgz_file))
	unzip_image(tgz_file, scene_dir)
    else:
	raise NotImplementedError('This image already exists at {}'.format(output_dir))

    return None


if __name__ == '__main__':
    pass

# ===============================================================================



# ===============================================================================
# Copyright 2017 dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================
import os
import re
import requests
from lxml import html
from pandas import DataFrame
from dateutil.rrule import rrule, DAILY
from datetime import datetime, timedelta


class OverpassNotFoundError(Exception):
    pass


class InvalidDateForSatelliteError(Exception):
    pass


class WebSiteOfflineError(Exception):
    pass


def verify_landsat_scene_exists(scene_string):
    if scene_string.startswith('LT5'):
        url_spec = '12266'
    elif scene_string.startswith('LE7'):
        url_spec = '13350'
    elif scene_string.startswith('LC8'):
        url_spec = '13400'
    else:
        raise NotImplementedError('Must choose a valid satellite to find url_spec')

    base = 'https://earthexplorer.usgs.gov/fgdc/'
    url = '{}{}/{}/'.format(base, url_spec, scene_string)

    r = requests.get(url)
    tree = html.fromstring(r.text)
    if r.status_code != 200:
        raise WebSiteOfflineError('USGS application unavailable.')
    string = tree.xpath('//pre/text()')

    split_str = string[0].split('\n')[5].split(':')
    title = [x.strip() for x in split_str]
    if len(title[1]) < 1:
        return False
    else:
        return True


def get_l5_overpass_data(path, row, date):
    if date > datetime(2013, 06, 01):
        raise ValueError('The date requested is after L5 deactivation')

    lat, lon = convert_lat_lon_wrs2pr(path, row, conversion_type='convert_pr_to_ll')

    url = 'https://cloudsgate2.larc.nasa.gov/cgi-bin/predict/predict.cgi'
    # submit form > copy POST data
    # use sat='SATELITE X' with a space
    num_passes = 30
    payload = dict(c='compute', sat='LANDSAT 5', instrument='0-0', res='9', month=str(date.month),
                   day=str(date.day),
                   numday=str(num_passes), viewangle='', solarangle='day', gif='track', ascii='element',
                   lat=str(lat),
                   lon=str(lon), sitename='Optional',
                   choice='track', year=str(date.year))

    r = requests.post(url, data=payload)
    tree = html.fromstring(r.text)
    head_string = tree.xpath('//table/tr[4]/td[1]/pre/b/font/text()')
    col = head_string[0].split()[8]
    ind = []
    zeniths = []

    for row in range(5, num_passes + 5):
        string = tree.xpath('//table/tr[{}]/td[1]/pre/font/text()'.format(row))
        l = string[0].split()
        dt_str = '{}-{}-{} {}:{}'.format(l[0], l[1], l[2], l[3], l[4])
        dt = datetime.strptime(dt_str, '%Y-%m-%d %H:%M')
        ind.append(dt)
        zenith = float(l[8])
        zeniths.append(zenith)
    df = DataFrame(zeniths, index=ind, columns=[col])

    print 'reference dtime overpass: {}'.format(df['zenith'].argmin())
    return df['zenith'].argmin()


def landsat_overpass_time(lndst_path_row, start_date, satellite):

    delta = timedelta(days=20)
    end = start_date + delta

    if satellite == 'LT5':

        if start_date > datetime(2013, 06, 01):
            raise InvalidDateForSatelliteError('The date requested is after L5 deactivation')

        reference_time = get_l5_overpass_data(lndst_path_row[0], lndst_path_row[1], start_date)
        return reference_time

    else:
        if satellite == 'LE7':
            sat_abv = 'L7'
        elif satellite == 'LC8':
            sat_abv = 'L8'

        base = 'https://landsat.usgs.gov/landsat/all_in_one_pending_acquisition/'
        for day in rrule(DAILY, dtstart=start_date, until=end):

            tail = '{}/Pend_Acq/y{}/{}/{}.txt'.format(sat_abv, day.year,
                                                      day.strftime('%b'),
                                                      day.strftime('%b-%d-%Y'))

            url = '{}{}'.format(base, tail)
            r = requests.get(url)

            for line in r.iter_lines():
                l = line.split()
                try:
                    if l[0] == str(lndst_path_row[0]):
                        if l[1] == str(lndst_path_row[1]):
                            # dtime is in GMT
                            time_str = '{}-{}'.format(day.year, l[2])
                            ref_time = datetime.strptime(time_str, '%Y-%j-%H:%M:%S')

                            return ref_time

                except IndexError:
                    pass
                except TypeError:
                    pass

        raise OverpassNotFoundError('Did not find overpass data, check your dates...')


def convert_lat_lon_wrs2pr(lat, lon, conversion_type='convert_ll_to_pr'):
    base = 'https://landsat.usgs.gov/landsat/lat_long_converter/tools_latlong.php'
    unk_number = 1490995492704

    if conversion_type == 'convert_ll_to_pr':

        full_url = '{}?rs={}&rsargs[]={}&rsargs[]={}&rsargs[]=1&rsrnd={}'.format(base, conversion_type,
                                                                                 lat, lon,
                                                                                 unk_number)
        r = requests.get(full_url)
        tree = html.fromstring(r.text)

        # remember to view source html to build xpath
        # i.e. inspect element > network > find GET with relevant PARAMS
        # > go to GET URL > view source HTML
        p_string = tree.xpath('//table/tr[1]/td[2]/text()')
        path = int(re.search(r'\d+', p_string[0]).group())

        r_string = tree.xpath('//table/tr[1]/td[4]/text()')
        row = int(re.search(r'\d+', r_string[0]).group())

        print 'path: {}, row: {}'.format(path, row)

        return path, row

    elif conversion_type == 'convert_pr_to_ll':

        full_url = '{}?rs={}&rsargs[]=\n' \
                   '{}&rsargs[]={}&rsargs[]=1&rsrnd={}'.format(base, conversion_type,
                                                               lat, lon, unk_number)

        r = requests.get(full_url)
        tree = html.fromstring(r.text)

        lat_string = tree.xpath('//table/tr[2]/td[2]/text()')
        lat = float(re.search(r'[+-]?\d+(?:\.\d+)?', lat_string[0]).group())

        lon_string = tree.xpath('//table/tr[2]/td[4]/text()')
        lon = float(re.search(r'[+-]?\d+(?:\.\d+)?', lon_string[0]).group())

        return lat, lon

    else:
        raise NotImplementedError('Must chose either convert_pr_to_ll or convert_ll_to_pr')


if __name__ == '__main__':
    pass

# ==================================================================================




import os

from setuptools import setup

os.environ['TRAVIS_CI'] = 'True'

try:
    from setuptools import setup

    setup_kwargs = {'entry_points': {'console_scripts': ['cloudsat=cloudsat.cloudsat:__main__']}}
except ImportError:
    from distutils.core import setup

    setup_kwargs = {'scripts': ['bin/cloudsat']}

tag = '0.0.2'

setup(name='CloudCRMSat',
      version=tag,
      description='Plugin Modificado para permitir download direto da Scena',
      setup_requires=['nose>=1.0'],
      py_modules=['cloudsat'],
      keywords='landsat download',
      author='Gustavo Junior',
      author_email='gustavorgjunior@gmail.com',
      platforms='Posix; MacOS X; Windows',
      packages=['cloudsat'],
      download_url='https://github.com/{}/{}/archive/{}.tar.gz'.format('brackbk', 'CloudCRMSat', tag),
      url='https://github.com/dgketchum',
      install_requires=['lxml==3.7.3', 'numpy==1.12.1', 'pandas==0.19.2',
                                                             'python-dateutil==2.6.0', 'pytz==2017.2',
                                                             'requests==2.13.0', 'six==1.10.0'],
      **setup_kwargs)

# ============= EOF ==============================================================



#!/usr/bin/env python

# ===============================================================================
# Copyright 2017 GustavoJunior
# ===============================================================================

import argparse
from datetime import datetime

import download


def create_parser():

    parser = argparse.ArgumentParser(prog='cloudsat', description='Download and unzip landsat data.')

    parser.add_argument('--file', help='Name Scene_ID', type=str)
    parser.add_argument('-o', '--output', help='Output directory')
    parser.add_argument('--credentials',
                        help='Path to a text file with USGS credentials with one space between <username password>')
    return parser


def main(args):
    if args:
        print '\nCloudCRM Download SCENE_ID...'
        print args
        download.down_usgs_by_id(args.file,args.output,args.credentials)
    else:
        raise NotImplementedError('Was not executed.')


def __main__():

    global parser
    parser = create_parser()
    args = parser.parse_args()
    exit(main(args))


if __name__ == '__main__':
    try:
        __main__()

    except KeyboardInterrupt:
        exit(1)

# ===============================================================================
